﻿using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.SearchModels;
using FurnitureAssemblyContracts.StoragesContracts;
using FurnitureAssemblyContracts.ViewModels;
using FurnitureAssemFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace FurnitureAssemFileImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataFileSingleton source;
        public OrderStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public List<OrderViewModel> GetFullList()
        {
            return source.Orders.Select(x => GetOrderViewModel(x)).ToList();
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && (!model.DateFrom.HasValue || !model.DateTo.HasValue))
            {
                return new();
            }
            if (model.Id.HasValue)
            {
                return source.Orders
                    .Where(x => x.Id.Equals(model.Id))
                    .Select(x => GetOrderViewModel(x))
                    .ToList();
            }
            return source.Orders
            .Where(x => model.DateFrom <= x.DateCreate && x.DateCreate <= model.DateTo)
           .Select(x => GetOrderViewModel(x))
           .ToList();
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            var order = source.Orders
            .FirstOrDefault(x =>
           (model.Id.HasValue && x.Id == model.Id));
            return GetOrderViewModel(order);
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = source.Orders.Count > 0 ? source.Orders.Max(x => x.Id) + 1 : 1;
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            source.Orders.Add(newOrder);
            source.SaveOrders();
            return GetOrderViewModel(newOrder);
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            source.SaveOrders();
            return GetOrderViewModel(order);
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            var element = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                source.Orders.Remove(element);
                source.SaveOrders();
                return GetOrderViewModel(element);
            }
            return null;
        }

        private OrderViewModel GetOrderViewModel(Order order)
        {
            OrderViewModel orderViewModel = order.GetViewModel;

            foreach (var furniture in source.Furnitures)
            {
                if (furniture.Id == order.FurnitureId)
                {
                    orderViewModel.FurnitureName = furniture.FurnitureName;
                }
            }
            return orderViewModel;
        }
    }
}
