﻿using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.BusinessLogicsContarcts;
using FurnitureAssemblyContracts.ViewModels;
using FurnitureAssemblyDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FurnitureAssembly
{
    public partial class FormReplenishmentShop : Form
    {

        private readonly List<ShopViewModel>? _listShops;
        private readonly List<FurnitureViewModel>? _listFurnitures;

        private IShopLogic _shopLogic;
        private ILogger _logger;

        public int Id
        {
            get
            {
                return Convert.ToInt32(comboBoxShop.SelectedValue);
            }
            set
            {
                comboBoxShop.SelectedValue = value;
            }
        }
      
        public int FurnitureId
        {
            get
            {
                return Convert.ToInt32(comboBoxFurniture.SelectedValue);
            }
            set
            {
                comboBoxFurniture.SelectedValue = value;
            }
        }

        public int Count
        {
            get { return Convert.ToInt32(textBoxCount.Text); }
            set
            { textBoxCount.Text = value.ToString(); }
        }
        public FormReplenishmentShop(ILogger<FormReplenishmentShop> logger, IShopLogic shopLogic, IFurnitureLogic furnitureLogic)
        {
            InitializeComponent();
            _logger = logger;
            _shopLogic = shopLogic;
            _listShops = shopLogic.ReadList(null);
            _listFurnitures = furnitureLogic.ReadList(null);
            if (_listShops != null)
            {
                comboBoxShop.DisplayMember = "ShopName";
                comboBoxShop.ValueMember = "Id";
                comboBoxShop.DataSource = _listShops;
                comboBoxShop.SelectedItem = null;
            }
            if (_listFurnitures != null)
            {
                comboBoxFurniture.DisplayMember = "FurnitureName";
                comboBoxFurniture.ValueMember = "Id";
                comboBoxFurniture.DataSource = _listFurnitures;
                comboBoxFurniture.SelectedItem = null;
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            if (comboBoxShop.SelectedValue == null)
            {
                MessageBox.Show("Выберите магазин", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxFurniture.SelectedValue == null)
            {
                MessageBox.Show("Выберите изделие", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Укажите количество поступившего изделия", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            try
            {
                _logger.LogInformation("Добавление в магазин с id = {ShopName} изделия: id = { FurnitureName} - { Count}", Id, FurnitureId, Count);
                var operationResult = _shopLogic.AddFurniture(new ShopBindingModel { Id = Id }, new FurnitureBindingModel { Id = FurnitureId }, Count);
                if (!operationResult)
                {
                    throw new Exception("Ошибка при пополнении магазина.  Дополнительная информация в логах.");
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при пополнении магазина c id = " + Id);
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
            DialogResult = DialogResult.OK;
            Close();
        }
    }
}
