﻿using FurnitureAssemblyBusinessLogic;
using FurnitureAssemblyContracts.BusinessLogicsContarcts;
using FurnitureAssemblyContracts.ViewModels;
using FurnitureAssemblyDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FurnitureAssembly
{
    public partial class FormSell : Form
    {
        private readonly IShopLogic _shopLogic;
        private readonly List<FurnitureViewModel>? _listFurnitures;

        public int Id
        {
            get
            {
                return Convert.ToInt32(comboBoxFurniture.SelectedValue);
            }
            set
            {
                comboBoxFurniture.SelectedValue = value;
            }
        }

        public IFurnitureModel? FurnitureModel
        {
            get
            {
                if (_listFurnitures == null)
                {
                    return null;
                }
                foreach (var elem in _listFurnitures)
                {
                    if (elem.Id == Id)
                    {
                        return elem;
                    }
                }
                return null;
            }
        }

        public int Count
        {
            get 
            { 
                return Convert.ToInt32(textBoxCount.Text); 
            }
            set
            {
                textBoxCount.Text = value.ToString();
            }
        }



        public FormSell(IShopLogic shopLogic, IFurnitureLogic furnitureLogic)
        {
            InitializeComponent();

            _shopLogic = shopLogic;

            _listFurnitures = furnitureLogic.ReadList(null);
            if (_listFurnitures != null)
            {
                comboBoxFurniture.DisplayMember = "FurnitureName";
                comboBoxFurniture.ValueMember = "Id";
                comboBoxFurniture.DataSource = _listFurnitures;
                comboBoxFurniture.SelectedItem = null;
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void buttonSell_Click(object sender, EventArgs e)
        {
            if (FurnitureModel == null)
            {
                MessageBox.Show("Выберите изделие", "Ошибка",
              MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (Count <= 0)
            {
                MessageBox.Show("Укажите количество изделия", "Ошибка",
              MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            if (!_shopLogic.Sell(new() { Id = FurnitureModel.Id }, Count))
            {
                MessageBox.Show("Не удалось продать изделие " + FurnitureModel.FurnitureName, "Ошибка",
             MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            MessageBox.Show("Продажа прошла успешно", "Сообщение",
               MessageBoxButtons.OK, MessageBoxIcon.Information);
            DialogResult = DialogResult.OK;
            Close();
        }
    }
}
