﻿using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.ViewModels;
using FurnitureAssemblyDataModels.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FurnitureAssemblyDatabaseImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        [Required]
        public string ShopName { get; private set; } = string.Empty;
        [Required]
        public string Address { get; private set; } = string.Empty;
        [Required]
        public DateTime DateOpening { get; private set; }

        private Dictionary<int, (IFurnitureModel, int)>? _furnitures = null;
        [NotMapped]
        public Dictionary<int, (IFurnitureModel, int)> Furnitures
        {
            get
            {
                if (_furnitures == null)
                {
                    _furnitures = ShopFurnitures
                    .ToDictionary(recSF => recSF.FurnitureId, recSF =>
                   (recSF.Furniture as IFurnitureModel, recSF.Count));

                }
                return _furnitures;
            }
            private set { }
        }

        public int MaxCount { get; private set; }

        [ForeignKey("ShopId")]
        public virtual List<ShopFurniture> ShopFurnitures { get; set; } = new();

        public static Shop? Create(FurnitureAssemblyDatabase context, ShopBindingModel model)
        {
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Address = model.Address,
                DateOpening = model.DateOpening,
                MaxCount = model.MaxCount,
                ShopFurnitures = model.Furnitures.Select(x => new ShopFurniture
                {
                    Furniture = context.Furnitures.First(y => y.Id == x.Key),
                    Count = x.Value.Item2
                }).ToList()
            };
        }

        public void Update(ShopBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            ShopName = model.ShopName;
            Address = model.Address;
            MaxCount = model.MaxCount;
            DateOpening = model.DateOpening;
            Furnitures = model.Furnitures;
        }

        public void UpdateFurnitures(FurnitureAssemblyDatabase context, ShopBindingModel model)
        {
            var shopFurnitures = context.ShopFurniture.Where(rec => rec.ShopId == model.Id).ToList();
            if (shopFurnitures != null && shopFurnitures.Count >= 0)
            { // удалили те, которых нет в модели
                context.ShopFurniture.RemoveRange(shopFurnitures.Where(rec => !model.Furnitures.ContainsKey(rec.FurnitureId)));
                context.SaveChanges();
                // обновили количество у существующих записей
                foreach (var updateComponent in shopFurnitures)
                {
                    updateComponent.Count = model.Furnitures[updateComponent.FurnitureId].Item2;
                    model.Furnitures.Remove(updateComponent.FurnitureId);
                }
                context.SaveChanges();
            }
            var shop = context.Shops.First(x => x.Id == Id);
            foreach (var pc in model.Furnitures)
            {
                context.ShopFurniture.Add(new ShopFurniture
                {
                    Shop = shop,
                    Furniture = context.Furnitures.First(x => x.Id == pc.Key),
                    Count = pc.Value.Item2
                });
                context.SaveChanges();
            }
            _furnitures = null;
        }

        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Address = Address,
            DateOpening = DateOpening,
            MaxCount = MaxCount,
            Furnitures = Furnitures
        };
    }
}
