﻿using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.SearchModels;
using FurnitureAssemblyContracts.StoragesContracts;
using FurnitureAssemblyContracts.ViewModels;
using FurnitureAssemblyListImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FurnitureAssemblyListImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataListSingleton _source;
        public OrderStorage()
        {
            _source = DataListSingleton.GetInstance();
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            for (int i = 0; i < _source.Orders.Count; ++i)
            {
                if (_source.Orders[i].Id == model.Id)
                {
                    var element = _source.Orders[i];
                    _source.Orders.RemoveAt(i);
                    return GetOrderViewModel(element);
                }
            }
            return null;
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }

            foreach (var order in _source.Orders)
            {
                if (model.Id.HasValue && order.Id == model.Id)
                {
                    return GetOrderViewModel(order);
                }
            }
            return null;
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && (!model.DateFrom.HasValue || !model.DateTo.HasValue))
            {
                return new();
            }
            var result = new List<OrderViewModel>();
            if (model.Id.HasValue)
            {
                foreach (var order in _source.Orders)
                {
                    if (order.Id.Equals(model.Id))
                    {
                        result.Add(GetOrderViewModel(order));
                        break;
                    }
                }
                return result;
            }
            foreach (var order in _source.Orders)
            {                
                if (model.DateFrom <= order.DateCreate && order.DateCreate <= model.DateTo)
                {
                    result.Add(GetOrderViewModel(order));
                }
            }
            return result;
        }

        public List<OrderViewModel> GetFullList()
        {
            var result = new List<OrderViewModel>();
            foreach (var order in _source.Orders)
            {
                result.Add(GetOrderViewModel(order));
            }
            return result;
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = 1;
            foreach (var order in _source.Orders)
            {
                if (model.Id <= order.Id)
                {
                    model.Id = order.Id + 1;
                }
            }
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            _source.Orders.Add(newOrder);
            return GetOrderViewModel(newOrder);
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            foreach (var order in _source.Orders)
            {
                if (order.Id == model.Id)
                {
                    order.Update(model);
                    return GetOrderViewModel(order);
                }
            }
            return null;
        }

        private OrderViewModel GetOrderViewModel(Order order)
        {
            OrderViewModel orderViewModel = order.GetViewModel;

            foreach (var furniture in _source.Furnitures)
            {
                if (furniture.Id == order.Id)
                {
                    orderViewModel.FurnitureName = furniture.FurnitureName;
                }
            }
            return orderViewModel;
        }
    }
}
